package org.openxma.dsl.generator.helper;

import org.openxma.dsl.core.model.DataTypeAndTypeParameter;
import org.openxma.dsl.core.model.EnumType;
import org.openxma.dsl.core.model.SimpleType;
import org.openxma.dsl.core.model.Validator;
import org.openxma.dsl.core.model.ValidatorReference;
import org.openxma.dsl.core.model.ValueModelType;
import org.openxma.dsl.dom.model.Attribute;
import org.openxma.dsl.generator.helper.style.BasicDataType;
import org.openxma.dsl.pom.model.CheckBox;
import org.openxma.dsl.pom.model.Combo;
import org.openxma.dsl.pom.model.ControlType;
import org.openxma.dsl.pom.model.CustomizeableField;
import org.openxma.dsl.pom.model.GuiElement;
import org.openxma.dsl.pom.model.ListBox;
import org.openxma.dsl.pom.model.Table;
import org.openxma.dsl.pom.model.TableCombo;
import org.openxma.dsl.pom.model.Text;

public class BasicTypeHelper {
    
    public static BasicType determineBasicType(Attribute attr) {
        /* First approach: determine BasicType from the name of the root validator */
        String validatorName = BasicTypeHelper.determineRootValidatorName(attr);
        if (ValidatorReference.STRING_VALIDATOR.equals(validatorName)) {
            return BasicType.STRING;
        }
        if (ValidatorReference.NUMBER_VALIDATOR.equals(validatorName)) {
            return BasicType.NUMBER;
        }
        if (ValidatorReference.BOOLEAN_VALIDATOR.equals(validatorName)) {
            return BasicType.BOOLEAN;
        }
        if (ValidatorReference.DATE_VALIDATOR.equals(validatorName)) {
            return BasicType.DATE;
        }
        if (ValidatorReference.TIME_STAMP_VALIDATOR.equals(validatorName)) {
            return BasicType.TIME_STAMP;
        }
        if (ValidatorReference.CUSTOM_VALIDATOR.equals(validatorName)) {
            return BasicType.CUSTOM;
        }
        if (ValidatorReference.I18N_ENUMERATION_VALIDATOR.equals(validatorName)) {
            return BasicType.I18N_ENUMERATION;
        }
        /* Second approach: determine BasicType from the model type of the root validator 
           (This approach is used for user defined validators.)  */
        ValueModelType modelType = BasicTypeHelper.determineRootValidatorModelType(attr);
        if (modelType != null) {
            switch (modelType.getValue()) {
            case ValueModelType.STRING_VALUE:
                return BasicType.STRING;
            case ValueModelType.NUMBER_VALUE:
                return BasicType.NUMBER;
            case ValueModelType.BOOLEAN_VALUE:
                return BasicType.BOOLEAN;
            case ValueModelType.DATE_VALUE:
                return BasicType.DATE;
            case ValueModelType.TIMESTAMP_VALUE:
                return BasicType.TIME_STAMP;
            case ValueModelType.ENUMERATION_VALUE:
                return BasicType.I18N_ENUMERATION;
            }
        }                                 
        return null;
    }   
    
    public static WidgetType determineWidgetType(GuiElement guiElement) {
        if (guiElement instanceof CustomizeableField) {
            return determineWidgetType((CustomizeableField)guiElement);
        } else if (guiElement instanceof Text) {
            return WidgetType.TEXT;
        } else if (guiElement instanceof CheckBox) {
            return WidgetType.CHECKBOX;            
        } else if (guiElement instanceof Combo) {
            return WidgetType.SIMPLECOMBO; 
        } else if (guiElement instanceof ListBox) {
            return WidgetType.LISTBOX;                
        } else if (guiElement instanceof Table) {
            return WidgetType.TABLE;            
        } else if (guiElement instanceof TableCombo) {
            return WidgetType.TABLECOMBO;            
        }
        throw new RuntimeException("Can not determine widget type of: "+guiElement);
    }    
    
    public static WidgetType determineWidgetType(CustomizeableField field) {
        /* Domain formatter overrules all */
        if (field.getFormat() != null) {
            String baseValidatorNameOfField = getBaseValidatorName(field.getFormat());
            if ("I18NEnumerationValidator".equals(baseValidatorNameOfField)) {
                return WidgetType.DOMAINCOMBO;
            }
        }
        /* Explicit control-type-specification of field */
        if (field.getControlType() != null) {
            if (useStaticDataLabel(field)) {
                return WidgetType.DATALABEL;
            } else if (field.getControlType().equals(ControlType.COMBO)) {
                return WidgetType.SIMPLECOMBO;
            } else if (field.getControlType().equals(ControlType.TABLECOMBO)) {
                return WidgetType.TABLECOMBO;
            } else if (field.getControlType().equals(ControlType.LISTBOX)) {
                return WidgetType.LISTBOX;
            } else if (field.getControlType().equals(ControlType.TEXT)) {
                return WidgetType.TEXT;
            }
        }
        /* Determine from attribute type */
        Attribute attr = field.getAttribute();
        DataTypeAndTypeParameter type = attr.getType();        
        if (isI18NEnumeration(type)) {
            return WidgetType.DOMAINCOMBO;
        }
        if (isBoolean(type)) {
            return WidgetType.CHECKBOX;
        }
        if (isDate(type)) {
            return WidgetType.DATEPICKER;
        }
        return WidgetType.TEXT;        
    }
    
    public static BasicDataType determineBasicDataType(CustomizeableField field) {
        Attribute attr = field.getAttribute();
        DataTypeAndTypeParameter type = attr.getType();
        if (type.getDataType() instanceof SimpleType) {
            SimpleType simpleType = (SimpleType)type.getDataType();
            String typeName = NameExtension.getFullyQualifiedName(simpleType,true);       
            // TODO change this dummy implementation to real type determination
            if ("java.math.BigDecimal".equals(typeName)) {
                typeName="BigDecimal";
            } else if ("java.util.Date".equals(typeName)) {
                typeName="Date";
            }
            return BasicDataType.valueOf(typeName);                   
        }
        return null;        
    }
    
    public static String determineRootValidatorName(Attribute attr) {
        ValidatorReference rootValidatorReference = determineRootValidator(attr);
        if (rootValidatorReference == null) {
            if (attr.getType().getDataType() instanceof EnumType) {
                return ValidatorReference.I18N_ENUMERATION_VALIDATOR;
            }
            throw new RuntimeException("Root validator is not defined for '"+attr.getType().getDataType().getName()+"'");
        }
        return determineRootValidatorName(rootValidatorReference);                
    }
    
    public static ValueModelType determineRootValidatorModelType(Attribute attr) {
        return determineRootValidatorModelType(attr.getType());
    }
        
    public static ValueModelType determineRootValidatorModelType(DataTypeAndTypeParameter dataTypeAndTypeParameter) {        
        ValidatorReference rootValidatorReference = determineRootValidator(dataTypeAndTypeParameter);   
        if (rootValidatorReference != null && rootValidatorReference.getValidator() != null) {
            return rootValidatorReference.getValidator().getValueModelType();           
        }  
        return null;
    }
        

    public static ValidatorReference determineRootValidator(Attribute attr) {
        return determineRootValidator(attr.getType());
    }        
        
    public static ValidatorReference determineRootValidator(DataTypeAndTypeParameter dataTypeAndTypeParameter) {        
        ValidatorReference validatorReference = dataTypeAndTypeParameter.getValidatorReferenceDefinition();
        if (validatorReference == null) {
            return null;
        }
        ValidatorReference rootValidatorReference = validatorReference.getRootValidatorReference();
        return rootValidatorReference;
    }
    
    public static String determineRootValidatorName(ValidatorReference rootValidatorReference) {
        String validatorName = rootValidatorReference.getValidator().getName();
        return validatorName;                
    }
    
    public static boolean useStaticDataLabel(CustomizeableField cf) {
        if (cf.getControlType() != null) {
            return cf.getControlType().equals(ControlType.LABEL);
        }
        return false;
    }
    
    public static boolean isI18NEnumeration(DataTypeAndTypeParameter dataTypeAndTypeParameter) {
        /* First approach */        
        String validatorName = getBaseValidatorName(dataTypeAndTypeParameter);
        if ("I18NEnumerationValidator".equals(validatorName)) {
            return true;
        }
        /* Second approach */
        ValueModelType modelType = determineRootValidatorModelType(dataTypeAndTypeParameter);
        if (modelType == ValueModelType.ENUMERATION) {
            return true;
        }
        return false;                 
    }

    public static boolean isBoolean(DataTypeAndTypeParameter dataTypeAndTypeParameter) {
        /* First approach */        
        String validatorName = getValidatorName(dataTypeAndTypeParameter);
        if ("BooleanValidator".equals(validatorName)) {
            return true;
        }
        /* Second approach */
        ValueModelType modelType = determineRootValidatorModelType(dataTypeAndTypeParameter);
        if (modelType == ValueModelType.BOOLEAN) {
            return true;
        }
        return false;         
    }

    public static boolean isDate(DataTypeAndTypeParameter dataTypeAndTypeParameter) {
        /* First approach */        
        String validatorName = getValidatorName(dataTypeAndTypeParameter);
        if ("DateValidator".equals(validatorName)) {
            return true;
        }
        /* Second approach */
        ValueModelType modelType = determineRootValidatorModelType(dataTypeAndTypeParameter);
        if (modelType == ValueModelType.DATE) {
            return true;
        }
        return false; 
    }
    
    public static String getBaseValidatorName(ValidatorReference validatorReference) {
        Validator val = validatorReference.getValidator();
        ValidatorReference superValRef = val.getSuperValidator();
        if (superValRef != null) {
            return getBaseValidatorName(superValRef);
        }
        return validatorReference.getValidator().getName();
    }

    public static String getBaseValidatorName(DataTypeAndTypeParameter dataTypeAndTypeParameter) {
        if (dataTypeAndTypeParameter.getDataType() instanceof EnumType) {
            return "I18NEnumerationValidator";
        }
        ValidatorReference validatorReference = SimpleTypeExtension.getValidatorReferenceDefinition(dataTypeAndTypeParameter);
        return getBaseValidatorName(validatorReference);
    }

    public static String getValidatorName(DataTypeAndTypeParameter dataTypeAndTypeParameter) {
        ValidatorReference validatorReference = SimpleTypeExtension.getValidatorReferenceDefinition(dataTypeAndTypeParameter);
        if (validatorReference != null) {
            return validatorReference.getValidator().getName();
        } else if (dataTypeAndTypeParameter.getDataType() instanceof EnumType) {
            return "I18NEnumerationValidator";
        } else {
            return null;
        }
    }

    public static DataTypeAndTypeParameter getRootSimpleType(DataTypeAndTypeParameter type) {
        if (type.getDataType() instanceof SimpleType) {
            org.openxma.dsl.core.model.impl.SimpleTypeImpl simpleType = (org.openxma.dsl.core.model.impl.SimpleTypeImpl) type.getDataType();
            DataTypeAndTypeParameter definition = simpleType.getTypeDefinition().getDefinition();
            if (definition != null)
                return getRootSimpleType(definition);
            return type;
        }
        return type;
    }
}
